// longExposure.cpp : Defines the entry point for the console application.
//


#include "stdafx.h"
#include <dshow.h>
#include <iostream>
#include <qedit.h>
#include <atlbase.h>
#include <fstream>
#include <math.h>
using namespace std;


bool WriteBMP(char* achName, BITMAPINFOHEADER *pBitmapInfoHeader, int nInfoSize, unsigned char* achData, int nBitmapSize)
{
	BITMAPFILEHEADER bmpFileHeader;

	fstream newFile(achName, ios::trunc| ios::in | ios::out | ios::binary);
	if (!newFile)
	{
		cout << "Open error on" << achName << endl;
		return(1);
	}

	bmpFileHeader.bfType = ((WORD) ('M' << 8) | 'B');
	bmpFileHeader.bfSize = nBitmapSize + nInfoSize + sizeof(bmpFileHeader);
	bmpFileHeader.bfReserved1 = 0;
	bmpFileHeader.bfReserved2 = 0;
	bmpFileHeader.bfOffBits = nInfoSize + sizeof(bmpFileHeader);

	newFile.write((char*)&bmpFileHeader, sizeof(bmpFileHeader));
	newFile.write((char*)pBitmapInfoHeader, nInfoSize);
	newFile.write((char*)achData, nBitmapSize);

	newFile.close();
	return TRUE;
}

int Power(int nBase, int nPower)
{
	int nResult = 1;
	for (int iii = 0; iii < nPower; iii++)
		nResult*= nBase;
	return nResult;
}

class CSampleGrabberCB : public ISampleGrabberCB
{
public:
	ISampleGrabber *pSampleGrabber;
	double* adCollector;
	int nFrames;
	BITMAPINFO BitmapInfo;
	int nPicBufSize;
	double dDecay;

	CSampleGrabberCB(void)
	{
		nPicBufSize = 0;
		adCollector = NULL;
		nFrames = 0;
		pSampleGrabber = NULL;
		dDecay = 1.00;
	}

	STDMETHOD(QueryInterface)(REFIID InterfaceIdentifier, VOID** ppvObject)
	{
		if (InterfaceIdentifier == __uuidof(ISampleGrabberCB))
		{
			*ppvObject = (ISampleGrabberCB**) this;
			return S_OK;
		}
		return E_NOINTERFACE;
	}
	STDMETHOD_(ULONG, AddRef)()
	{
		return 2;
	}
	STDMETHOD_(ULONG, Release)()
	{
		return 1;
	}


	STDMETHODIMP SampleCB(double dTime, IMediaSample *pms)
	{
		return 0;
	}
	STDMETHODIMP BufferCB(double dSampleTime, byte *pBuffer, long nBufSize)
	{
		AM_MEDIA_TYPE MediaType;
		ZeroMemory(&MediaType, sizeof(MediaType));
		HRESULT hr = pSampleGrabber->GetConnectedMediaType(&MediaType);
		if (FAILED(hr))
		{
			cout << "Can't get media type" << endl;
			return hr;
		}
		VIDEOINFOHEADER *pVideoHeader = (VIDEOINFOHEADER*)MediaType.pbFormat;
		if (pVideoHeader == NULL)
		{
			cout << "pVideoHeader = NULL" << endl;
			return hr;
		}

		ZeroMemory(&BitmapInfo, sizeof(BitmapInfo));
		CopyMemory(&BitmapInfo.bmiHeader, &(pVideoHeader->bmiHeader), sizeof(BITMAPINFOHEADER));

		if (BitmapInfo.bmiHeader.biBitCount != 24)
		{
			cout << "Not 24 bit, tell Brian to add 32 bit support" << endl;
			return hr;
		}


		if (nFrames == 0)
		{
			adCollector = new double [nBufSize];
			nPicBufSize = nBufSize;
			ZeroMemory(adCollector, nBufSize * sizeof(double));
		}
	
		nFrames++;

//		for (int iii = 0; iii < nBufSize; iii++)
//			adCollector[iii] = adCollector[iii]*dDecay + pBuffer[iii];

		for (int iii = 0; iii < nBufSize; iii++)
			adCollector[iii] += pow((double)pBuffer[iii], 16);




		cout << dSampleTime << endl;
		return 0;
	}
};




void main(int argc, char* argv[])
{
	cout << "================ Long Exposure ==================" << endl;
	cout << "By Brian Coventry" << endl;

    IGraphBuilder *pGraph = NULL;
    IMediaControl *pControl = NULL;
    IMediaEvent   *pEvent = NULL;
	IBaseFilter *pGrabberBaseFilter = NULL;
	ISampleGrabber *pSampleGrabber = NULL;


 
    HRESULT hr = CoInitialize(NULL);
    if (FAILED(hr))
    {
        printf("ERROR - Could not initialize COM library");
        return;
    }

 
    hr = CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, 
                        IID_IGraphBuilder, (void **)&pGraph);
    if (FAILED(hr))
    {
        printf("ERROR - Could not create the Filter Graph Manager.");
        return;
    }

	AM_MEDIA_TYPE mt;
	hr = CoCreateInstance(CLSID_SampleGrabber, NULL, CLSCTX_INPROC_SERVER,
		IID_IBaseFilter, (void**)&pGrabberBaseFilter);
	if (FAILED(hr))
		exit(1);
	pGrabberBaseFilter->QueryInterface(IID_ISampleGrabber, (void**)&pSampleGrabber);
	if (pSampleGrabber == NULL)
		exit(1);
	hr = pGraph->AddFilter(pGrabberBaseFilter,L"Grabber");
	if(FAILED(hr))
		exit(1);

	ZeroMemory(&mt, sizeof(AM_MEDIA_TYPE));
	mt.majortype = MEDIATYPE_Video;
	mt.subtype = MEDIASUBTYPE_RGB24;
	mt.formattype = FORMAT_VideoInfo;
	hr = pSampleGrabber->SetMediaType(&mt);
	if (FAILED(hr))
		exit(1);

    hr = pGraph->QueryInterface(IID_IMediaControl, (void **)&pControl);
    hr = pGraph->QueryInterface(IID_IMediaEvent, (void **)&pEvent);

  
	LPWSTR lFileName = SysAllocStringLen(NULL, MAX_PATH);
	MultiByteToWideChar(CP_ACP, 0, argv[1], -1, lFileName, MAX_PATH);
	hr = pGraph->RenderFile(lFileName, NULL);
	SysFreeString(lFileName);
	if (FAILED(hr))
		exit(1);

	CSampleGrabberCB CB;
	CB.pSampleGrabber = pSampleGrabber;
	hr = pSampleGrabber->SetCallback(&CB, 1);
	CB.dDecay = 1.00;


	CComQIPtr<IMediaSeeking> pSeek = pControl;
	LONGLONG nDuration;
	hr = pSeek->GetDuration(&nDuration);
	if (FAILED(hr))
		exit(1);
	REFERENCE_TIME rtStart = 0;
	REFERENCE_TIME rtStop = nDuration-1;

	hr = pSeek->SetPositions(&rtStart, AM_SEEKING_AbsolutePositioning,
		&rtStop, AM_SEEKING_AbsolutePositioning);

	if (FAILED(hr))
		exit(1);


	CComQIPtr<IVideoWindow> pVideoWindow = pGraph;

	hr = pVideoWindow->put_AutoShow(OAFALSE);

    hr = pControl->Run();
	if (FAILED(hr))
		exit(1);

    long evCode;
    hr = pEvent->WaitForCompletion(INFINITE, &evCode);
	if (FAILED(hr))
		exit(1);




//	AM_MEDIA_TYPE MediaType;
//	ZeroMemory(&MediaType,sizeof(MediaType));
//	hr = pSampleGrabber->GetConnectedMediaType(&MediaType);
//	if (FAILED(hr))
//		exit(1);
//
//	VIDEOINFOHEADER *pVideoHeader = (VIDEOINFOHEADER*)MediaType.pbFormat;
//	if (pVideoHeader == NULL)
//		exit(1);
//
//	BITMAPINFO BitmapInfo;
//	ZeroMemory(&BitmapInfo, sizeof(BitmapInfo));
//	CopyMemory(&BitmapInfo.bmiHeader, &(pVideoHeader->bmiHeader), sizeof(BITMAPINFOHEADER));
//	
//	byte *buffer = NULL;
//
//	long nBufSize = NULL;
//	hr = pSampleGrabber->GetCurrentBuffer(&nBufSize, NULL);
//	if (FAILED(hr))
//		exit(1);
//
//	buffer = (BYTE*)CoTaskMemAlloc(nBufSize);
//
//	hr = pSampleGrabber->GetCurrentBuffer(&nBufSize, (long*)buffer);
//	if (FAILED(hr))
//		exit(1);

	unsigned char * achBuffer = new unsigned char [CB.nPicBufSize];
	for (int iii = 0; iii < CB.nPicBufSize; iii++)
		achBuffer[iii] = (unsigned char)sqrt(sqrt(sqrt(sqrt(CB.adCollector[iii] / CB.nFrames))));

//	double dLargest = 0;
//	for (int iii = 0; iii < CB.nPicBufSize; iii++)
//		if (dLargest < CB.adCollector[iii])
//			dLargest = CB.adCollector[iii];
//
//	double dFactor = 255/dLargest;
//	for (int iii = 0; iii < CB.nPicBufSize; iii++)
//		achBuffer[iii] = (unsigned char)(dFactor*CB.adCollector[iii]);


	char achName[20] = "file.bmp";
	WriteBMP(argv[2], &(CB.BitmapInfo.bmiHeader), CB.BitmapInfo.bmiHeader.biSize, achBuffer, CB.nPicBufSize);


	pSampleGrabber->Release();
	pGrabberBaseFilter->Release();
	pControl->Release();
	pEvent->Release();
	pGraph->Release();
	CoUninitialize();
}
